/*
    char.c
    Jim Hall <jhall@freedos.org>

    (This software is distributed under the 0-clause BSD license)

    Permission to use, copy, modify, and/or distribute this software for
    any purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
    WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
    AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
    DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
    OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
    TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
*/

#include <stdio.h>
#include <stdlib.h>

#include <ctype.h> /* for isdigit */

int
is_int(char *str)
{
    int pos;

    for (pos = 0; str[pos]; pos++) {
	if ( !isdigit(str[pos]) ) {
	    /* found a non-integer digit character */
	    return 0; /* false */
	}
    }

    /* no non-int digits found .. return true */
    /* if empty str, then pos will be zero, so can return that to indicate
	this is not an integer */

    return pos;
}

int
main(int argc, char **argv)
{
    int dec;
    int i;

    /* if no options, print an error and quit */

    if (argc == 1) {
	fputs("char: print an ascii character (this program is public domain)\n", stderr);
	fputs("usage: char [n ..]\n", stderr);
	fputs("   n = decimal value (0-255)\n", stderr);

	return 1;
    }

    /* print the arguments as ascii characters */

    for (i = 1; i < argc; i++) {
	dec = atoi(argv[i]);

	/* only print valid characters 0-255 */

	if (!is_int(argv[i])) {
	    fprintf(stderr, "warning - not a number: %s\n", argv[i]);
	    putchar('?');
	}
	else if ((dec < 0) || (dec > 255)) {
	    fprintf(stderr, "warning - outside range 0-255: %s\n", argv[i]);
	    putchar('?');
	}
	else {
	    putchar(dec);
	}
    }

    return 0;
}
